﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Graphics.Display;
using Windows.Graphics.Imaging;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using RenderChartToBitmap.Annotations;

namespace RenderChartToBitmap
{
    public class MainPageViewModel : INotifyPropertyChanged
    {
        private DelegateCommand<UIElement> renderChartCommand;
        private string status = "Click button to render and save Chart";
        private ImageSource resultImageSource;

        public MainPageViewModel()
        {
            
        }

        public string Status
        {
            get { return status; }
            set { status = value; OnPropertyChanged(); }
        }

        private ObservableCollection<double> chartData;
        public ObservableCollection<double> ChartData
        {
            get { return chartData ?? (chartData = new ObservableCollection<double> { 20, 30, 50, 10, 60, 40, 20, 80 }); }
            set { chartData = value; OnPropertyChanged();}
        }

        public ImageSource ResultImageSource
        {
            get { return resultImageSource; }
            set { resultImageSource = value; OnPropertyChanged(); }
        }
        
        public DelegateCommand<UIElement> RenderChartCommand
        {
            get { return renderChartCommand ?? (renderChartCommand = new DelegateCommand<UIElement>(async (e) => await RenderElement(e))); }
        }

        private async Task RenderElement(UIElement element)
        {

            Status = $"starting render...";

            var rtb = new RenderTargetBitmap();
            await rtb.RenderAsync(element);

            Status = $"render complete, setting Image source in UI...";

            // Show the image in the UI
            ResultImageSource = rtb;

            Status = $"Rednering done! Click button to save to file";
            
            var pixelBuffer = await rtb.GetPixelsAsync();

            var savePicker = new FileSavePicker();
            savePicker.DefaultFileExtension = ".png";
            savePicker.FileTypeChoices.Add(".png", new List<string> { ".png" });
            savePicker.SuggestedStartLocation = PickerLocationId.PicturesLibrary;
            savePicker.SuggestedFileName = "MyChart.png";

            var saveFile = await savePicker.PickSaveFileAsync();

            if (saveFile == null)
                return;

            // Encode the image to the selected file on disk (docs https://msdn.microsoft.com/en-us/library/windows/apps/windows.graphics.imaging.bitmapencoder.aspx )
            using (var fileStream = await saveFile.OpenAsync(FileAccessMode.ReadWrite))
            {
                var encoder = await BitmapEncoder.CreateAsync(BitmapEncoder.PngEncoderId, fileStream);

                encoder.SetPixelData(
                    BitmapPixelFormat.Bgra8,
                    BitmapAlphaMode.Ignore,
                    (uint)rtb.PixelWidth,
                    (uint)rtb.PixelHeight,
                    DisplayInformation.GetForCurrentView().LogicalDpi,
                    DisplayInformation.GetForCurrentView().LogicalDpi,
                    pixelBuffer.ToArray());

                await encoder.FlushAsync();
            }

            Status = $"File Saved to {saveFile.Path}";
        }

        #region INPC

        
        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}
